import random
import string

import pytest

import confik
from confik import ConfikError


def get_random_key():
    return "".join(random.choice(string.ascii_uppercase) for _ in range(20))


def test_read_variable_from_env():
    env = confik.get("VARIABLE")
    assert env == "VALUE", "VARIABLE is equal to {v}".format(v=env)


def test_read_variable_from_os():
    env = confik.get("PATH")
    assert env, "env is {v}".format(v=env)


def test_get_non_existent_key():
    random_key = get_random_key()

    with pytest.raises(ConfikError):
        confik.get(random_key)


def test_default_value():
    random_key = get_random_key()

    env = confik.get(random_key, "DEFAULT_VALUE")
    assert (
        env == "DEFAULT_VALUE"
    ), "env is not equal to default value, current value is {v}".format(v=env)


def test_default_factory():
    random_key = get_random_key()
    env = confik.get(random_key, default_factory=lambda x: "DEFAULT_FACTORY_VALUE")

    assert env == "DEFAULT_FACTORY_VALUE"


def test_cast_to_csv():
    env = confik.get("VARIABLE_WITH_MULTI_VALUES", cast=confik.csv)
    assert len(env) == 4
    assert env == ["V1", "V2", "V3", "V4"]


def test_csv_elements_has_no_trailing_or_preceding_space():
    env = confik.get("VARIABLE_WITH_MULTI_VALUES", cast=confik.csv)

    for i, element in enumerate(env):
        assert bool(
            element.find(" ")
        ), "element at index {i} has space {element}".format(
            i=i,
            element=element,
        )


def test_read_truthy_bools():
    for i in range(1, 4):
        env = confik.get("VARIABLE_WITH_TRUE_VALUE_{i}".format(i=i), cast=bool)
        assert env is True, "Expected True, got {}".format(env)


def test_read_falsy_bools():
    for i in range(1, 4):
        env = confik.get("VARIABLE_WITH_FALSE_VALUE_{i}".format(i=i), cast=bool)
        assert env is False, "Expected False, got {}".format(env)


def test_read_integer():
    env = confik.get("VARIABLE_WITH_INTEGER", cast=int)
    assert isinstance(env, int)


def test_read_not_bool_as_bool():
    with pytest.raises(AssertionError):
        env = confik.get("PATH", cast=bool)


def test_confik_boolean_same_as_python_bool():
    with pytest.raises(AssertionError):
        env = confik.get("PATH", cast=confik.boolean)



from collections.abc import Callable, Iterable

from confik.exceptions import ConfikError
from confik.proxies import EnvMappingProxy
from confik.utils import boolean


class ConfikParser:
    proxy_class = EnvMappingProxy

    def __init__(self, path=".", *args, **kwargs):
        self.source = self.proxy_class(*args, path=path, **kwargs)

    def validate_params(
        self,
        key,
        cast,
        default,
        default_factory,
        choices,
        raise_exception,
    ):
        if choices is None:
            choices = set()

        assert isinstance(key, str), "key must be a string"
        assert isinstance(cast, Callable) or cast is None, "cast must be a callable"
        assert (
            isinstance(default_factory, Callable) or default_factory is None
        ), "Must provide a callable for default_factory"
        assert not all(
            [default, default_factory]
        ), "default and default_factory parameters are mutually exclusive, must only provide one of them."

        assert isinstance(choices, Iterable), "choices must be an iterable"
        assert isinstance(raise_exception, bool), "raise_exception must be a boolean"

        if default and choices:
            assert (
                default in choices
            ), "default value is not in the list of provided choices"

    def get(
        self,
        key,
        default=None,
        cast=None,
        default_factory=None,
        choices=tuple(),
        raise_exception=True,
    ):
        self.validate_params(
            key, cast, default, default_factory, choices, raise_exception
        )

        env = self.source.get(key, default)

        if env is None:
            if default_factory:
                return default_factory(key)
            elif raise_exception:
                raise ConfikError(
                    "{key} variable can't be of type None".format(key=key)
                )

        if choices:
            assert env in choices, "{key}={value} is not in choices={choices}".format(
                key=key,
                value=env,
                choices=", ".join(choices),
            )

        if cast:
            try:
                if cast is bool:
                    return boolean(env)
                return cast(env)
            except ValueError:
                raise ConfikError(
                    "value {v} can't be casted into {c}".format(v=env, c=cast.__name__)
                )

        return env



from typing import List

_truthy_values = ("True", "true", "1")
_falsy_values = ("False", "false", "0")
_allowed_bool_values = (*_truthy_values, *_falsy_values)


def validate_choices(value, choices):
    assert (
        value in choices
    ), "{value} is not in the predefined choices {choices}".format(
        value=value, choices=", ".join(choices)
    )


def csv(value: str) -> List[str]:
    return list(element.strip() for element in value.split(","))


def boolean(value: str) -> bool:
    assert value in _allowed_bool_values, "value of {v} is not a bool".format(v=value)
    return value in _truthy_values



import contextlib
import os
from pathlib import Path


class MapConfigToMappingProxy:
    def get_mapping(self, path):
        """
        Should return a mapping from the path given
        :param path: str
        :return: Mapping
        """
        raise NotImplementedError


class EnvMappingProxy(MapConfigToMappingProxy):
    def get_mapping(self, path):
        mapping = {}

        with contextlib.suppress(FileNotFoundError):
            with open(path, "r") as f:
                for line in f.readlines():
                    entry = line.strip().split("=", 1)

                    if len(entry) == 1:
                        entry.append("")

                    name, value = entry
                    mapping[name] = value.replace('"', "")

        return mapping

    def __init__(self, path=Path("."), *args, **kwargs):
        assert isinstance(path, (str, Path)), "unsupported path type {t}".format(
            t=type(path)
        )

        if isinstance(path, str):
            path = Path(path)

        if path.suffix != ".env":
            path = path / ".env"

        self.environ = {
            **os.environ,
            **self.get_mapping(path),
        }

    def get(self, key, default):
        return self.environ.get(key, default)



from .exceptions import ConfikError
from .parsers import ConfikParser
from .utils import boolean, csv

confik = ConfikParser()


def get(
    key,
    default=None,
    cast=None,
    default_factory=None,
    choices=tuple(),
    raise_exception=True,
):
    """An interface to confik.get(...)

    :param key: The key name to get from environment
    :param default: The value to return if the key was not found
    :param cast: A function to call on the value after it's found
    :param default_factory: A function to call with the key to return a value if the value was not found
    :param choices: A choices list to ensure that the value is in it
    :param raise_exception: Raise exceptions if the value was not found
    :return: str
    """
    return confik.get(key, default, cast, default_factory, choices, raise_exception)



from distutils.core import setup

setup(
    name="confik",
    version="1.0.5",
    description="Config parser for environment variables",
    author="LeOndaz",
    author_email="ahmeddark369@gmail.com",
    url="https://github.com/LeOndaz/confik",
    packages=["confik"],
)



from pathlib import Path

from confik import ConfikParser


def test_create_confik_parser_with_str_path():
    assert ConfikParser("."), "Can't create parser with path as string"


def test_create_confik_parser_with_path():
    assert ConfikParser(Path(".")), "Can't create parser with path as pathlib.Path"
